#version 100

precision mediump float;

varying vec2 TextureCoordinate;

uniform float viewWidth;
uniform float viewHeight;
uniform float startX;
uniform float startY;
uniform float endX;
uniform float endY;
uniform float strokeWidth;
uniform float capRadius;
uniform float ringWidth;

const float TOLERANCE = 0.01;

const vec4 transparentColor = vec4(0.0, 0.0, 0.0, 0.0);
const vec4 strokeColor = vec4(1.0, 0.35, 0.12, 1.0);
const vec4 capColor = vec4(1.0, 0.35, 0.12, 1.0);
const vec4 ringColor = vec4(1.0, 1.0, 1.0, 1.0);

// draw line segment from A to B
float segment(vec2 P, vec2 A, vec2 B, float r)
{
    vec2 g = B - A;
    vec2 h = P - A;
    float d = length(h - g * clamp(dot(g, h) / dot(g,g), 0.0, 1.0));
    return smoothstep(r, 0.5*r, d);
}

void main() {
    float outerRadius = capRadius + ringWidth;
    float halfStrokeWidth = strokeWidth * 0.5;

    // 将纹理坐标点转换成对应view的size坐标点
    vec2 pointVec = vec2(TextureCoordinate.x * viewWidth, TextureCoordinate.y * viewHeight);
    vec2 startPoint = vec2(startX, startY);
    vec2 endPoint = vec2(endX, endY);
    //起点
    float pointToStartLength = length(pointVec - startPoint);
    if(pointToStartLength < capRadius){
        gl_FragColor = capColor;
        return;
    }else if(pointToStartLength < outerRadius){
        gl_FragColor = ringColor;
        return;
    }
    //终点
    float pointToEndLength = length(pointVec - endPoint);
    if(pointToEndLength < capRadius){
        gl_FragColor = capColor;
        return;
    }else if(pointToEndLength < outerRadius){
        gl_FragColor = ringColor;
        return;
    }

    //经过验证该方法绘制的线段比下面的方案绘制的线段更平滑
    vec2 A = vec2(startPoint.x/viewWidth, startPoint.y/viewHeight);
    vec2 B = vec2(endPoint.x/viewWidth, endPoint.y/viewHeight);
    float r = strokeWidth/max(viewWidth, viewHeight);
    float intensity = segment(TextureCoordinate, A, B, r);
    gl_FragColor = mix(transparentColor, strokeColor, intensity);


//    //水平线
//    float centerx, centery, left, right, bottom, up;
//    if(abs(startPoint.y - endPoint.y) <= TOLERANCE){
//        centery = (startPoint.y + endPoint.y) * 0.5;
//        bottom = centery - halfStrokeWidth;
//        up = centery + halfStrokeWidth;
//        left = min(startPoint.x, endPoint.x);
//        right = max(startPoint.x, endPoint.x);
//        if(pointVec.x >= left && pointVec.x <= right && pointVec.y >= bottom && pointVec.y <= up){
//            gl_FragColor = strokeColor;
//            return;
//        }
//    }
//    //垂直线
//    if(abs(startPoint.x - endPoint.x) <= TOLERANCE){
//        centerx = (startPoint.x + endPoint.x) * 0.5;
//        left = centerx - halfStrokeWidth;
//        right = centerx + halfStrokeWidth;
//        bottom = min(startPoint.y, endPoint.y);
//        up = max(startPoint.y, endPoint.y);
//        if(pointVec.x >= left && pointVec.x <= right && pointVec.y >= bottom && pointVec.y <= up){
//            gl_FragColor = strokeColor;
//            return;
//        }
//    }
//    //带斜率的直线
//    left = min(startPoint.x, endPoint.x);
//    right = max(startPoint.x, endPoint.x);
//    bottom = min(startPoint.y, endPoint.y);
//    up = max(startPoint.y, endPoint.y);
//    if(pointVec.x >= left && pointVec.x <= right && pointVec.y >= bottom && pointVec.y <= up){
//        float k = (endPoint.y - startPoint.y)/(endPoint.x - startPoint.x);
//        float c = endPoint.y - k * endPoint.x;
//        vec2 line = vec2((pointVec.y - c)/k, pointVec.x * k + c);
//        if((pointVec.y >= line.y - halfStrokeWidth && pointVec.y <= line.y + halfStrokeWidth) || (pointVec.x >= line.x - halfStrokeWidth && pointVec.x <= line.x + halfStrokeWidth)){
//            gl_FragColor = strokeColor;
//            return;
//        }
//    }
//
//    gl_FragColor = transparentColor;
//    return;
}

