#version 100

precision mediump float;

varying vec2 TextureCoordinate;

uniform float strokeWidth;
uniform float alpha;
uniform float progress;

const float PI = 3.141592653589793;
const float TWO_PI = 6.283185307179586;
const vec2 UNIT_Y = vec2(0.0, -1.0);

const vec4 transparentColor = vec4(0.0, 0.0, 0.0, 0.0);
const vec4 backgroundColor = vec4(0.0, 0.0, 0.0, 0.72);
const vec4 strokeColor = vec4(1.0, 1.0, 1.0, 1.0);

void main() {
    // 将纹理坐标点转换成对应view的size坐标点
    vec2 pointVec = TextureCoordinate;
    // 计算进度的弧度值
    float progressArc = progress * TWO_PI;
    // 计算圆心和半径
    float outerCircleRadius = 0.5;
    float innerCircleRadius = outerCircleRadius - strokeWidth;
    // 由圆心向定点方向的向量
//    vec2 pointVec = circleCenter-point;
    float lenOfPointVec = length(pointVec);
    // 在圆环以外直接设置为透明色
    if (lenOfPointVec < innerCircleRadius || lenOfPointVec > outerCircleRadius) {
        gl_FragColor = transparentColor * alpha;
        return;
    }
    float pointCos;
    // 在圆环以内需要根据进度进行判断
    if(progressArc >= PI){ //进度条起始点在右侧
        if(pointVec.x < 0.0){ //y轴左侧全部填白色
            gl_FragColor = strokeColor * alpha;
            return;
        }
        float elapsedArc = TWO_PI - progressArc;
        float progressCos = cos(elapsedArc);
        pointCos = dot(UNIT_Y, pointVec)/lenOfPointVec;
        if(pointCos > progressCos){
            gl_FragColor = transparentColor * alpha;
            return;
        }
        gl_FragColor = strokeColor * alpha;
        return;
    }
    //进度条起始点在左侧
    if(pointVec.x >= 0.0){ //y轴左侧全部填白色
        gl_FragColor = transparentColor * alpha;
        return;
    }
    float remainingArc = progressArc;
    float remainingCos = cos(remainingArc);
    pointCos = dot(UNIT_Y, pointVec)/lenOfPointVec;
    if(pointCos <= remainingCos){
        gl_FragColor = transparentColor * alpha;
        return;
    }
    gl_FragColor = strokeColor * alpha;
}

